#!/usr/bin/env perl
use strict;
use AnyEvent;
use Errno qw(EAGAIN EINTR);

my $cv  = AnyEvent->condvar;
my $sig = AnyEvent->signal(
    signal => 'INT',
    cb => sub {
        print "got signal INT\n";
        $cv->send;
    }
);

# ここでは標準入力を使用するので、明示的に非同期に設定します
my $fh = \*STDIN;
AnyEvent::Util::fh_nonblocking($fh, 1);

my $io; $io = AnyEvent->io(
    fh => $fh,
    poll => "r",
    cb => sub {
        my $len = sysread( $fh, my $buf, 8192 );
        if ($len > 0) {
            print "read '$buf'\n";
        } elsif (defined $len) {
            # ハンドルがcloseされた
            undef $io;
        } elsif ($! != EAGAIN && $! != EINTR) {
            # よくないエラー
            undef $io;
            die "An error occurred: $!";
        }
    }
);

print "標準入力に文字列を入力してください。\n",
    "  (スクリプトを中断するにはCtrl-Cを押してください)\n";

$cv->recv;
